#ifndef __WORKER__
#define __WORKER__
#include <semaphore.h>
#include "list.h"
#include "task.h"

typedef struct task_queue {
    list_t head;    //list of tasks
    sem_t lock;     //mutex
    sem_t nr_task;  //number of tasks    

    void    (*destroy)(struct task_queue *self);
    void    (*add    )(struct task_queue *self, task_t *task);
    task_t *(*remove )(struct task_queue *self);
} task_queue_t;

typedef struct worker {
    int id;                 //worker id
    pthread_t tid;          //thread id of the worker
    int done;               //if true exit the thread
    task_queue_t *task_q;   //task queue

    void  (*destroy)(struct worker *self);
    void  (*start  )(struct worker *self);
    void  (*stop   )(struct worker *self);
    void *(*work   )(void *_self); //thread func
} worker_t;

extern task_queue_t *make_task_queue();
extern worker_t *make_worker(task_queue_t *task_q);

#endif